@@ -156,6 +156,8 @@ struct _ClockData {
         GtkWidget *custom_entry;
         gboolean   custom_format_shown;
 
+	gboolean   can_handle_format_12;
+
 	guint listeners [N_GCONF_PREFS];
 };
 
@@ -856,17 +858,24 @@ task_activated_cb (GtkTreeView       *view,
                    GtkTreeViewColumn *column,
                    ClockData         *cd)
 {
-        GtkTreeIter iter;
-        char       *uri;
+        GtkTreeIter  iter;
+        GtkTreePath *child_path;
+        char        *uri;
 
-        gtk_tree_model_get_iter (GTK_TREE_MODEL (cd->tasks_model), &iter, path);
+        child_path = gtk_tree_model_filter_convert_path_to_child_path (cd->tasks_filter, path);
+
+        gtk_tree_model_get_iter (GTK_TREE_MODEL (cd->tasks_model), &iter, child_path);
         gtk_tree_model_get (GTK_TREE_MODEL (cd->tasks_model), &iter,
                             TASK_COLUMN_URL, &uri, -1);
 
-        if (uri)
+        if (uri) {
                 gnome_url_show_on_screen (uri,
 					  gtk_widget_get_screen (cd->applet),
 					  NULL);
+        }
+
+        g_free (uri);
+        gtk_tree_path_free (child_path);
 
         return TRUE;
 }
@@ -924,10 +933,10 @@ appointment_pixbuf_cell_data_func (GtkTreeViewColumn *column,
 }
 
 static int
-compare_priority  (GtkTreeModel *model,
-                   GtkTreeIter  *a,
-                   GtkTreeIter  *b,
-                   gpointer      user_data)
+compare_tasks  (GtkTreeModel *model,
+                GtkTreeIter  *a,
+                GtkTreeIter  *b,
+                gpointer      user_data)
 {
         int priority_a;
         int priority_b;
@@ -946,8 +955,21 @@ compare_priority  (GtkTreeModel *model,
 		return -1;
 	else if (priority_a > priority_b)
 		return 1;
-	else
-		return 0;
+	else {
+                GTime due_time_a, due_time_b;
+
+                gtk_tree_model_get (model, a,
+				    TASK_COLUMN_DUE_TIME, &due_time_a, -1);
+                gtk_tree_model_get (model, b,
+				    TASK_COLUMN_DUE_TIME, &due_time_b, -1);
+
+                if (due_time_a < due_time_b)
+                        return -1;
+                else if (due_time_a > due_time_b)
+                        return 1;
+                else
+                        return 0;
+        }
 }
 
 static GtkWidget *
@@ -993,7 +1015,7 @@ create_task_list (ClockData  *cd,
 
                 gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (cd->tasks_model),
                                                  TASK_COLUMN_PRIORITY,
-                                                 compare_priority,
+                                                 compare_tasks,
                                                  NULL, NULL);
 
                 gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (cd->tasks_model),
@@ -1216,15 +1238,29 @@ create_appointment_list (ClockData  *cd,
 }
 
 static void
-calendar_day_activated (ClockData *cd)
+calendar_day_activated (ClockData   *cd,
+			GtkCalendar *calendar)
 {
-        /* FIXME: should be able to launch the editor for
-         *        the specific day
-         */
-        calendar_client_launch_editor (cd->client,
-                                       CALENDAR_EVENT_APPOINTMENT,
-                                       gtk_widget_get_screen (cd->calendar),
-                                       NULL);
+	unsigned int  day;
+	unsigned int  month;
+	unsigned int  year;
+	char         *command_line;
+	GError       *error;
+
+	gtk_calendar_get_date (GTK_CALENDAR (cd->calendar),
+			       &year, &month, &day);
+
+	command_line = g_strdup_printf ("evolution "
+					"calendar:///?startdate=%.4d%.2d%.2d",
+					year, month + 1, day);
+
+	if (!gdk_spawn_command_line_on_screen (gtk_widget_get_screen (cd->calendar),
+					       command_line, &error)) {
+		g_printerr ("Cannot launch calendar: %s\n", error->message);
+		g_error_free (error);
+	}
+
+	g_free (command_line);
 }
 
 static void
@@ -1319,7 +1355,7 @@ add_appointments_and_tasks (ClockData *cd,
         update_frame_visibility (cd->appointment_list, GTK_TREE_MODEL (cd->appointments_model));
 
         switch (cd->orient) {
-        case PANEL_APPLET_ORIENT_DOWN:
+        case PANEL_APPLET_ORIENT_UP:
                 gtk_box_pack_start (GTK_BOX (vbox), cd->appointment_list, TRUE, TRUE, 0);
                 gtk_box_pack_start (GTK_BOX (vbox), cd->task_list, TRUE, TRUE, 0);
                 break;
@@ -1409,7 +1445,7 @@ create_calendar (ClockData *cd,
         gtk_calendar_select_day (GTK_CALENDAR (cd->calendar), tm->tm_mday);
 
 	switch (cd->orient) {
-        case PANEL_APPLET_ORIENT_DOWN:
+        case PANEL_APPLET_ORIENT_UP:
                 add_appointments_and_tasks (cd, vbox);
                 gtk_box_pack_start (GTK_BOX (vbox), cd->calendar, TRUE, FALSE, 0);
                 break;
@@ -1476,7 +1512,7 @@ position_calendar_popup (ClockData *cd,
 	 * I expected.
 	 */
 	switch (cd->orient) {
-	case PANEL_APPLET_ORIENT_LEFT:
+	case PANEL_APPLET_ORIENT_RIGHT:
 		x += button_w;
 		if ((y + h) > monitor.y + monitor.height)
 			y -= (y + h) - (monitor.y + monitor.height);
@@ -1487,7 +1523,7 @@ position_calendar_popup (ClockData *cd,
 			gravity = GDK_GRAVITY_NORTH_WEST;
 
 		break;
-	case PANEL_APPLET_ORIENT_RIGHT:
+	case PANEL_APPLET_ORIENT_LEFT:
 		x -= w;
 		if ((y + h) > monitor.y + monitor.height)
 			y -= (y + h) - (monitor.y + monitor.height);
@@ -1498,7 +1534,7 @@ position_calendar_popup (ClockData *cd,
 			gravity = GDK_GRAVITY_NORTH_EAST;
 
 		break;
-	case PANEL_APPLET_ORIENT_UP:
+	case PANEL_APPLET_ORIENT_DOWN:
 		y += button_h;
 		if ((x + w) > monitor.x + monitor.width)
 			x -= (x + w) - (monitor.x + monitor.width);
@@ -1506,7 +1542,7 @@ position_calendar_popup (ClockData *cd,
 		gravity = GDK_GRAVITY_NORTH_WEST;
 
 		break;
-	case PANEL_APPLET_ORIENT_DOWN:
+	case PANEL_APPLET_ORIENT_UP:
 		y -= h;
 		if ((x + w) > monitor.x + monitor.width)
 			x -= (x + w) - (monitor.x + monitor.width);
@@ -1657,9 +1693,9 @@ create_clock_widget (ClockData *cd)
 	cd->orient = panel_applet_get_orient (PANEL_APPLET (cd->applet));
 
 	/* Initialize label orientation */
-	if (cd->orient == PANEL_APPLET_ORIENT_LEFT)
+	if (cd->orient == PANEL_APPLET_ORIENT_RIGHT)
 		gtk_label_set_angle (GTK_LABEL (cd->clockw), 90);
-	else if (cd->orient == PANEL_APPLET_ORIENT_RIGHT)
+	else if (cd->orient == PANEL_APPLET_ORIENT_LEFT)
 		gtk_label_set_angle (GTK_LABEL (cd->clockw), 270);
 	else
 		gtk_label_set_angle (GTK_LABEL (cd->clockw), 0);
@@ -1686,9 +1722,9 @@ applet_change_orient (PanelApplet       *applet,
 {
         cd->orient = orient;
         
-        if (cd->orient == PANEL_APPLET_ORIENT_LEFT)
+        if (cd->orient == PANEL_APPLET_ORIENT_RIGHT)
                 gtk_label_set_angle (GTK_LABEL (cd->clockw), 90);
-        else if (cd->orient == PANEL_APPLET_ORIENT_RIGHT)
+        else if (cd->orient == PANEL_APPLET_ORIENT_LEFT)
                 gtk_label_set_angle (GTK_LABEL (cd->clockw), 270);
         else
                 gtk_label_set_angle (GTK_LABEL (cd->clockw), 0);
@@ -1698,48 +1734,24 @@ applet_change_orient (PanelApplet       *applet,
         update_popup (cd);
 }
 
-static void
-applet_change_background (PanelApplet               *applet,
-			  PanelAppletBackgroundType  type,
-			  GdkColor                  *color,
-			  GdkPixmap                 *pixmap,
-			  ClockData                 *cd)
-{
-	GtkRcStyle *rc_style;
-	GtkStyle   *style;
-
-	/* reset style */
-	gtk_widget_set_style (GTK_WIDGET (cd->applet), NULL);
-	rc_style = gtk_rc_style_new ();
-	gtk_widget_modify_style (GTK_WIDGET (cd->applet), rc_style);
-	gtk_rc_style_unref (rc_style);
-
-	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		break;
-	case PANEL_COLOR_BACKGROUND:
-		gtk_widget_modify_bg (GTK_WIDGET (cd->applet),
-				      GTK_STATE_NORMAL, color);
-		break;
-	case PANEL_PIXMAP_BACKGROUND:
-		style = gtk_style_copy (GTK_WIDGET (cd->applet)->style);
-		if (style->bg_pixmap[GTK_STATE_NORMAL])
-			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
-		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
-		gtk_widget_set_style (GTK_WIDGET (cd->applet), style);
-		g_object_unref (style);
-		break;
-	}
-}
-
-
 /* this is when the panel size changes */
 static void
-applet_change_pixel_size (PanelApplet *applet,
-			  gint         size,
-			  ClockData   *cd)
+applet_change_pixel_size (PanelApplet   *applet,
+			  GtkAllocation *allocation,
+			  ClockData     *cd)
 {
-	cd->size = size;
+	int new_size;
+
+        if (cd->orient == PANEL_APPLET_ORIENT_LEFT ||
+	    cd->orient == PANEL_APPLET_ORIENT_RIGHT)
+		new_size = allocation->width;
+	else
+		new_size = allocation->height;
+
+	if (cd->size == new_size)
+		return;
+
+	cd->size = new_size;
 
         unfix_size (cd);
 	update_timeformat (cd);
@@ -1875,6 +1887,7 @@ try_config_tool (GdkScreen  *screen,
 					 GTK_BUTTONS_OK,
 					 _("Failed to launch time configuration tool: %s"),
 					 err->message);
+	g_error_free (err);
 		
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_widget_destroy), NULL);
@@ -1945,8 +1958,13 @@ format_changed (GConfClient  *client,
 		return;
 
 	value = gconf_value_get_string (entry->value);
-	if (!gconf_string_to_enum (format_type_enum_map, value, &new_format) ||
-	    new_format == clock->format)
+	if (!gconf_string_to_enum (format_type_enum_map, value, &new_format))
+		return;
+
+	if (!clock->can_handle_format_12 && new_format == CLOCK_FORMAT_12)
+		new_format = CLOCK_FORMAT_24;
+
+	if (new_format == clock->format)
 		return;
 
 	clock->format = new_format;
@@ -2252,7 +2270,7 @@ fill_clock_applet (PanelApplet *applet)
 	cd->showdate = panel_applet_gconf_get_bool (applet, KEY_SHOW_DATE, &error);
 	if (error) {
 		g_error_free (error);
-		/* if on a small screen don't show data by default */
+		/* if on a small screen don't show date by default */
 		if (gdk_screen_width () <= 800)
 			cd->showdate = FALSE;
 		else
@@ -2265,6 +2283,10 @@ fill_clock_applet (PanelApplet *applet)
 
 	cd->timeformat = NULL;
 
+	cd->can_handle_format_12 = (clock_locale_format () == CLOCK_FORMAT_12);
+	if (!cd->can_handle_format_12 && cd->format == CLOCK_FORMAT_12)
+		cd->format = CLOCK_FORMAT_24;
+
 	create_clock_widget (cd);
 
 	gtk_container_set_border_width (GTK_CONTAINER (cd->applet), 0);
@@ -2283,16 +2305,13 @@ fill_clock_applet (PanelApplet *applet)
 			  G_CALLBACK (applet_change_orient),
 			  cd);
 
-	/* similiar to the above in semantics*/
 	g_signal_connect (G_OBJECT (cd->applet),
-			  "change_size",
+			  "size-allocate",
 			  G_CALLBACK (applet_change_pixel_size),
 			  cd);
 
-	g_signal_connect (G_OBJECT (cd->applet),
-			  "change_background",
-			  G_CALLBACK (applet_change_background),
-			  cd);
+	panel_applet_set_background_widget (PANEL_APPLET (cd->applet),
+					    GTK_WIDGET (cd->applet));
 
 	panel_applet_setup_menu_from_file (PANEL_APPLET (cd->applet),
 					   NULL,
@@ -2430,7 +2449,10 @@ set_format_cb (GtkComboBox *combo,
         ClockFormat format;
 
 	/* valid values begin from 1 */
-        format = gtk_combo_box_get_active (combo) + 1;
+	if (cd->can_handle_format_12)
+		format = gtk_combo_box_get_active (combo) + 1;
+	else
+		format = gtk_combo_box_get_active (combo) + 2;
 
         update_properties_for_format (cd, combo, format);
 
@@ -2571,7 +2593,8 @@ display_properties_dialog (BonoboUIComponent *uic,
         combo = gtk_combo_box_new_text ();
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
 
-        gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("12 hour"));
+	if (cd->can_handle_format_12)
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("12 hour"));
         gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("24 hour"));
         gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("UNIX time"));
         gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Internet time"));
@@ -2638,7 +2661,13 @@ display_properties_dialog (BonoboUIComponent *uic,
         update_properties_for_format (cd, GTK_COMBO_BOX (combo), cd->format);
 
 	/* valid values begin from 1 */
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), cd->format - 1);
+	if (cd->can_handle_format_12)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
+					  cd->format - 1);
+	else
+		gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
+					  cd->format - 2);
+
         g_signal_connect (combo, "changed",
                           G_CALLBACK (set_format_cb), cd);
 
@@ -2659,20 +2688,10 @@ display_help_dialog (BonoboUIComponent *uic,
 		     const gchar       *verbname)
 {
 	GError *error = NULL;
-	static GnomeProgram *applet_program = NULL;
 
-	if (!applet_program) {
-		int argc = 1;
-		char *argv[2] = { "clock" };
-		applet_program = gnome_program_init ("clock", VERSION,
-						      LIBGNOME_MODULE, argc, argv,
-						      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
-	}
-
-	gnome_help_display_desktop_on_screen (
-			applet_program, "clock", "clock",NULL,
-			gtk_widget_get_screen (cd->applet),
-			&error);
+	gnome_help_display_desktop_on_screen (NULL, "clock", "clock", NULL,
+					      gtk_widget_get_screen (cd->applet),
+					      &error);
 	if (error) {
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (NULL,
@@ -2737,6 +2756,10 @@ display_about_dialog (BonoboUIComponent *uic,
 	g_signal_connect (G_OBJECT(cd->about), "destroy",
 			  (GCallback)gtk_widget_destroyed, &cd->about);
 	
+	g_signal_connect (cd->about, "response",
+			  G_CALLBACK (gtk_widget_destroy),
+			  NULL);
+
 	gtk_widget_show (cd->about);
 }
 
