@@ -27,7 +27,7 @@
 
 G_BEGIN_DECLS
 
-void panel_session_init           (const char *argv0);
+void panel_session_init           (void);
 void panel_session_request_logout (void);
 
 G_END_DECLS
